@@ -69,7 +69,7 @@ class HistoricalStatistics:
         self.case_history = case_history
         self.death_history = death_history
         self.recovery_history = recovery_history
-        self.province = province or 'None'
+        self.province = province or None
 
     
 class JhuCsseStatistics:
@@ -82,4 +82,22 @@ class JhuCsseStatistics:
         self.deaths = deaths
         self.recoveries = recoveries
         self.latitude = _lat
-        self.longitude = _long
\ No newline at end of file
+        self.longitude = _long
+
+
+class ContinentStatistics:
+    def __init__(self, name, cases, deaths, recoveries, critical, active, tests, today_cases,
+                 today_deaths, cases_per_million, deaths_per_million, tests_per_million, updated):
+        self.name = name
+        self.cases = cases
+        self.deaths = deaths
+        self.recoveries = recoveries
+        self.critical = critical
+        self.active = active
+        self.tests = tests
+        self.today_cases = today_cases
+        self.today_deaths = today_deaths
+        self.cases_per_million = cases_per_million
+        self.deaths_per_million = deaths_per_million
+        self.tests_per_million = tests_per_million
+        self.updated = updated
\ No newline at end of file
